; *********************** X P L O D E . L S P ***************************
; XPLODE.LSP  operates like  "EXPLODE" but  it will  explode 2D blocks of
; unequal  scales that are in the WCS.   Nested blocks  are not exploded,
; but they will be equally scaled and look correct.  They may not reside
; in the WCS plan however.
; The  Z  scale  is  changed  to  0  during the execution of the program.      
; AutoCad's PROJECT.LSP is used by this program and its limitations apply.
; Entities are projected onto the WCS.
;
; The program works by rescaling the block to equal scales and tilting it
; into the z  axis to create  an image that  appears the  same as before.
; E.G.  When a  circle is  tilted  it looks  like an  ellipse. It is then
; exploded  and projected  back into  the world plane  using two slightly
; modified modules of  PROJECT.LSP.  PROJECT.LSP operation is unaffected.
;
; To get a better idea of what is happening, issue the command
; VPOINT 1,-1,1 before running XPLODE.LSP
;
; Walter Warren, Compuserve 73750,3535
;
; I know this works on my system with R12c2 and it was originally written
; for R10.

; If PROJECT.LSP isn't loaded, do so.
(vmon)
(COMMAND "UNDO" "M")
(if (not c:project) 
  (progn 
    (prompt "\nLoading PROJECT.LSP ") (load "project")
  )
)
(prompt "\nLoading XPLODE.LSP ")
;
; Modified version of PROJECT.LSP's (proj-ent) function
(defun proj-e (/ ctr setlength ename elist n-of-ents)
 (prompt "\nExtrusion and mesh information will not be projected. ")
 (setq ctr 0)
 (setq setlength (sslength entities))
 (prompt "\nWorking .")
 (while (setq ename (ssname entities ctr))
   (prompt ".")
   (setq elist (entget ename)
         etype (cdr (assoc 0 elist))
   )
   (cond ((or (eq etype "LINE") (eq etype "3DLINE")) (lines-pro))
         ((eq etype "CIRCLE") (circ-pro))
         ((eq etype "ARC") (arc-pro))
         ((eq etype "POLYLINE" ) (pline-pro))
         ((eq etype "3DFACE") (face-pro))
         ((eq etype "POINT") (point-pro))
         ((or (eq etype "TRACE") (eq etype "SOLID")) (solid-pro))
         (T (setq reject-ent (ssadd ename reject-ent)))
   )
   (setq ctr (+ ctr 1))
 )
 (setq n-of-ents (sslength reject-ent))
 (princ (strcat "\n" (itoa n-of-ents) " entities not projected"))
 (redraw-rej)
)
;
;  Modified version of PROJECT.LSP main program as a function
(defun proj ()
  (mapcar 'setvar
          '("cmdecho" "blipmode" "expert" "flatland")
          '(0 0 1 0)
  )
  (setq reject-ent (ssadd))
  (setq entset (ssadd))
  (proj-e)
  (moder)
  (setq *error* olderr)
  (princ)
)
;
(defun dxf (code elst)
 (cdr (assoc code elst))
)
;
(defun c:xplode (/ en en1 ent ang x y z ins e0 uins alph)
 (setq cm (getvar "cmdecho"))
 (setvar "cmdecho" 0)
 (setq en1 (entsel "\nSelect Block to Xplode: ")
       ent (entget (car en1))
       ang (rtd (dxf 50 ent))
       x (dxf 41 ent)
       y (dxf 42 ent)
       z (dxf 43 ent)
       ins (dxf 10 ent)
       e0 (entlast)
       en (entnext e0)
 )
 (while (not (null en))                ; find the last entity
  (setq e0 en)
  (setq en (entnext e0))
 )
 (cond
  ((< y x)                               ; X scale greater than Y
   (entmod
    (setq ent (subst (cons 42 x) (cons 42 y) ent)
          ent (subst (cons 43 x) (cons 43 z) ent)
    )
   )
   (command "ucsicon" "on"
            "ucs" "z" ang
            "ucs" "x" 90
            "ucs" "y" 90
   )
   (setq uins (trans ins 0 1))
   (if (< y 0) (command "rotate" en1 "" uins "180"))
   (setq alph (* (atan (/ (sqrt (- (* x x) (* y y))) y)) (/ 180 pi)))
   (command "rotate" en1 "" uins alph)   ; ^ Formula for Arccos (y/x)
   (command "ucs" "w")
   (command "explode" en1)   ; explode the entity
;
   (setq entities (ssadd))
   (while (entnext e0) (ssadd (setq e0 (entnext e0)) entities))
   (setq lay (getvar "clayer"))
   (command "layer" "s" (dxf 8 ent) "")
   (proj)
   (command "erase" entities "r" reject-ent "")
   (redraw)
   (command "layer" "s" lay "")
;
  )
;
  ((< x y)                               ; X scale less than Y
   (entmod
    (setq ent (subst (cons 41 y) (cons 41 x) ent)
          ent (subst (cons 43 y) (cons 43 z) ent)
    )
   )
   (command "ucsicon" "on"
            "ucs" "z" ang
            "ucs" "x" 90
   )
   (setq uins (trans ins 0 1))
   (if (< x 0) (command "rotate" en1 "" uins "180"))
   (setq alph (* (atan (/ (sqrt (- (* y y) (* x x))) x)) (/ 180 pi)))
   (command "rotate" en1 "" uins alph)   ; ^ Formula for Arccos (x/y)
   (command "ucs" "w")
   (command "explode" en1)    ; explode the entity
;
   (setq entities (ssadd))
   (while (entnext e0) (ssadd (setq e0 (entnext e0)) entities))
   (command "layer" "s" (dxf 8 ent) "")
   (proj)
   (command "erase" entities "r" reject-ent "")
   (redraw)
   (command "layer" "s" lay "")
;
  )
;
  ((= y x)                               ; X scale equal to Y
   (setq ent (subst (cons 43 x) (cons 43 z) ent))
   (entmod ent)
   (command "explode" en1)
   (setq entities (ssadd))
   (while (entnext e0) (ssadd (setq e0 (entnext e0)) entities))
   (command "chprop" entities ""
            "c"   "bylayer"
            "lt"  "bylayer"
            "la"  (dxf 8 ent) "")
  )
 )
 (prompt "\nBlock has been Xploded")
 (command "ucsicon" "off")
 (setvar "cmdecho" cm)
 (setq *error* olderr)
 (princ)
)
